export declare type F<T> = (...args: any[]) => T;
export declare class Transaction {
    private static id;
    readonly id: number;
    readonly hint: string;
    readonly snapshot: number;
    readonly error: any;
    private _snapshot;
    private changeset;
    private pending;
    private sealed;
    private _error;
    private resultPromise;
    private resultResolve;
    private resultReject;
    private conflicts;
    private refreshing;
    constructor(hint?: string);
    static readonly current: Transaction | undefined;
    run<T>(func: F<T>, ...args: any[]): T;
    wrap<T>(func: F<T>): F<T>;
    commit(): void;
    sealToCommit(): Transaction;
    discard(error?: any): Transaction;
    waitForFinish(): Promise<void>;
    finished(): boolean;
    undo(): void;
    redo(): void;
    static run<T>(hint: string, func: F<T>, ...args: any[]): T;
    static runAsync<T>(hint: string, func: F<Promise<T>>, ...args: any[]): Promise<T>;
    private _run;
    private sealAndRethrowErrorIfNeeded;
    private static seal;
    private checkConflictsResolved;
    private tryResolveConflicts;
    private freezeChangeset;
    private performCommit;
    private performDiscard;
    private acquirePromise;
    static _acquireSnapshotNumber(t: Transaction): number;
    static _addToChangeset(t: Transaction, handle: any, record: any): void;
    static _getFromChangeset(t: Transaction, handle: any): any;
    static _wrap<T>(t: Transaction, c: Cache | undefined, increment: boolean, decrement: boolean, func: F<T>): F<T>;
}
export declare class StoreParams {
    readonly name?: string;
    readonly idProperty?: string;
    readonly isObjectWithId?: (obj: object) => boolean;
    readonly twoWaySync?: (incomingPatches: object[], outgoingPatches: object[]) => void;
}
export declare class Store {
    readonly params: StoreParams;
    constructor(params?: StoreParams);
    static dispose(obj: object): void;
}
declare class Handle {
    private static id;
    readonly id: number;
    readonly store: Store;
    readonly proxy: any;
    private committed;
    private writable;
    constructor(store: Store, proxy: any);
    getCommitted(): Record;
    getForTran(t: Transaction, c: Cache | undefined, prop: string): Record | undefined;
    getForRead(prop: string): Record;
    acquireForWrite(prop: string): Record;
    tryAcquireForWrite(prop: string, value: any): Record | undefined;
    static applyChangeset(changeset: Map<Handle, Record>): void;
}
declare type PropertyMask<T, V> = {
    [P in keyof T]: V;
};
declare class Record {
    readonly handle: Handle;
    readonly previous: Record | undefined;
    readonly transaction: Transaction;
    readonly data: any;
    readonly changemask: PropertyMask<any, boolean>;
    readonly conflictmask: PropertyMask<any, Record>;
    readonly observers: PropertyMask<any, Set<Cache>>;
    constructor(handle: Handle, previous: Record | undefined, tran: Transaction, data: object);
    freezeData(): void;
    subscribeForPropBeingRead(prop: string, cache: Cache | undefined): void;
    invalidateObservers(refreshing: Cache[]): number;
    determineConflicts(): boolean;
    invalidateObserversIfAny(prop: string, observers: Set<Cache> | undefined, refreshing?: Cache[]): number;
    hint(): string;
}
declare class MethodInfo {
    readonly isTran: boolean;
    readonly isAsync: boolean;
    readonly name: string;
    readonly body: (...args: any[]) => any;
    constructor(isTran: boolean, name: string, body: F<any>);
}
declare class Cache {
    readonly owner: Record;
    readonly method: MethodInfo;
    readonly wrapper: (...args: any[]) => any;
    readonly observables: PropertyMask<any, Set<Record>>;
    args: any[];
    value: any;
    invalidator: Record | undefined;
    dirty: Record | undefined;
    error: any;
    keepfresh: boolean;
    margin: number;
    constructor(owner: Record, method: MethodInfo, keepfresh: boolean, margin: number);
    run<T>(func: F<T>, ...args: any[]): T;
    wrap<T>(func: F<T>): F<T>;
    ensureFresh(): void;
    tryInvalidateBy(invalidator: Record, prop: string, refreshing?: Cache[]): number;
    markDirty(invalidator: Record, prop: string): void;
    private wrapAsTran;
    private wrapAsCache;
    private renew;
    private renewAsync;
    private hint;
}
export interface Status {
    value: any;
    outdated: boolean;
    error: any;
    keepfresh: boolean;
}
export declare function statusof(method: F<any>): Status;
export declare function keepfresh(value: boolean, method: F<any>, ...args: any[]): void;
export declare function dispose(obj: object): void;
export declare const transactional: typeof tran;
export declare function tran(target: object, prop?: string, descriptor?: TypedPropertyDescriptor<F<any>>): any;
export declare function cache(ctor: object, prop: string, descriptor: TypedPropertyDescriptor<F<any>>): any;
export {};
